#include "measurement.h"

/* kp szlessge, kp magassga, vanishing pointok mtrixkoordintinak beadsa */
Measurement::Measurement(int w, int h, int vap1_x, int vap1_y, int vap2_x, int vap2_y) {

	width = w;
	height = h;
	
	vp1_x = vap1_x;
	vp1_y = h - vap1_y;
	vp2_x = vap2_x;
	vp2_y = h - vap2_y;

	if (vp1_x == vp2_x) vp1_x++;	//koordinata modosito, hogy az iranytangens ne legyen vegtelen
	if (vp1_y == vp2_y) vp1_y++;	//koordinata modosito, hogy az iranytangens ne legyen vegtelen

}

	
void Measurement::setRefData( int ireftop_x, int ireftop_y, int irefdown_x, int irefdown_y, float frefsize) {

	reftop_x = ireftop_x;
	reftop_y = height - ireftop_y;
	refdown_x = irefdown_x;
	refdown_y = height - irefdown_y;
	refsize = (frefsize < 0) ? -frefsize : frefsize;

	if (reftop_x == refdown_x) reftop_x++;	//koordinata modosito, hogy az iranytangens ne legyen vegtelen
	if (reftop_y == refdown_y) reftop_y++;	//koordinata modosito, hogy az iranytangens ne legyen vegtelen
}


void Measurement::setMeasureCoordinates(
				int imtop_x, //mrend tetejnek x koord.
				int imtop_y, //mrend tetejnek y koord.
				int imdown_x, //mrend aljnak x koord.
				int imdown_y) { //mrend aljnak y koord.

	mtop_x = imtop_x;
	mtop_y = height - imtop_y;
	mdown_x = imdown_x;
	mdown_y = height - imdown_y;

	if (mtop_x == mdown_x) mtop_x++;	//koordinata modosito, hogy az iranytangens ne legyen vegtelen
	if (mtop_y == mdown_y) mtop_y++;	//koordinata modosito, hogy az iranytangens ne legyen vegtelen

}

double Measurement::calculateSize() { //mrend objektum mrett meghatroz metdus

	double Mtegy;		//talppontokat sszekt egyenlet irnytangense
	double btegy;		//talppontokat sszekt egyenlet b paramtere

	double Mvanish;		//vanishing line irnytangense
	double bvanish;		//vanishing line b paramtere

	double Mrefvanish;	//ref. teteje s vanishing line ltal meghatrozott egyenes irnytangense;
	double brefvanish;	//ref. teteje s vanishing line ltal meghatrozott egyenes b paramtere

	double talpmetsz_x;	// a talppontok ltal meghatrozott egyenes s a vanishing line x metszspontja
	double talpmetsz_y;	// a talppontok ltal meghatrozott egyenes s a vanishing line y metszspontja

	double Mmerendo;		//mrend obj. egyenesnek irnytangense	
	double bmerendo;		//mrend obj. egyenesbek b paramtere

	double mvanref_x;	//mererendo obj. egyenes - s vanishing line-ref. obj egyenes x metszespontja
	double mvanref_y;	//mererendo obj. egyenes - s vanishing line-ref. obj egyenes y metszespontja

	double meret_merendo;			//merendo obj. merete pixelben
	double meret_merendo_refvanish;	//merendo obj. talpa s vansihing-reftop egyenes metszspontjnak mrete pixelben
	double meret_ref;				//referencia objektum mrete pixelben


	/* Talppontokat meghatarozo egyenes M es b parameterenek meghatarozasa (y = Mx + b)*/

	Mtegy = (double)((double)(mdown_y - refdown_y) / (double)(mdown_x - refdown_x));	// talppontokat sszekt egyenes irnytangense
	btegy = (double)((refdown_y * (mdown_x - refdown_x)) - (refdown_x * (mdown_y - refdown_y))) / (mdown_x - refdown_x);	// s b paramtere

	Mvanish = (double)(vp2_y - vp1_y) / (vp2_x - vp1_x);	//vanishing line iranytangense
	bvanish = (double)((vp1_y * (vp2_x - vp1_x)) - (vp1_x * (vp2_y - vp1_y))) / (vp2_x - vp1_x);	//vanishing line b parametere
	
	if ( Mtegy == Mvanish ) { //akkor nincs metszspont a vansihing line-al, egyszer arnyszmts

		meret_merendo = (double) sqrt(((mdown_x - mtop_x)* (mdown_x - mtop_x)) + ((mdown_y - mtop_y)*(mdown_y - mtop_y)));
		meret_ref = (double) sqrt(((refdown_x - reftop_x)* (refdown_x - reftop_x)) + ((refdown_y - reftop_y)*(refdown_y - reftop_y)));
		return refsize * meret_merendo / meret_ref;
	
	}

	/* objektumok talppontjt sszekt s a vanishing line egyenesek metszspontjai */

	talpmetsz_y = (double)((Mvanish * btegy) - (Mtegy * bvanish)) / (Mvanish - Mtegy);
	talpmetsz_x = (double)(talpmetsz_y - btegy) / Mtegy;

	if (!(talpmetsz_x < 2147483647 && talpmetsz_x > -2147483647)) {

		/* amennyiben a metszspont tl messze van, akkor feltesszk, h a kt obj. egyms mellett ll */
		meret_merendo = (double) sqrt(((mdown_x - mtop_x)* (mdown_x - mtop_x)) + ((mdown_y - mtop_y)*(mdown_y - mtop_y)));
		meret_ref = (double) sqrt(((refdown_x - reftop_x)* (refdown_x - reftop_x)) + ((refdown_y - reftop_y)*(refdown_y - reftop_y)));
		return refsize * meret_merendo / meret_ref;

	}

	if ((talpmetsz_y == mdown_y) || (refdown_y == talpmetsz_y)) {

		/* ekkor valamelyik (pontosan egy) objektum a vegtelen tavoli sikon van, a meres nem vegezheto el. */
		return -1;
	}

	/*megvan a vanishing line s a talppontokat sszekt egyenes metszspontja */

	/* most meghatrozzuk a fenti metszspont s a ref. obj teteje ltal meghat. egyenes egyenlett */

	Mrefvanish = (double) (talpmetsz_y - reftop_y) / (talpmetsz_x - reftop_x);
	brefvanish = (double) ((reftop_y * (talpmetsz_x - reftop_x)) - (reftop_x * (talpmetsz_y - reftop_y))) / (talpmetsz_x - reftop_x);
	
	/* most meghatrozzuk a mrend obj. egyenlett */
	
	Mmerendo = (double) (mdown_y - mtop_y) / (mdown_x - mtop_x);
	bmerendo = (double) ((mtop_y * (mdown_x - mtop_x)) - (mtop_x * (mdown_y - mtop_y))) / (mdown_x - mtop_x);

	/* meghatrozzuk a mrend objektum s a talpmetsz - reftop egyenes metszspontjt */

	mvanref_y = (double) ((Mrefvanish * bmerendo) - (Mmerendo * brefvanish)) / (Mrefvanish - Mmerendo);
	mvanref_x = (double) (mvanref_y - brefvanish) / Mrefvanish;

	/* kiszmoljuk a mreteket pixelben (arnyszmtshoz kell) */

	meret_merendo = (double) sqrt(((mdown_x - mtop_x)* (mdown_x - mtop_x)) + ((mdown_y - mtop_y)*(mdown_y - mtop_y)));
	meret_merendo_refvanish = (double) sqrt(((mdown_x - mvanref_x)* (mdown_x - mvanref_x)) + ((mdown_y - mvanref_y)*(mdown_y - mvanref_y)));

	
	if (!((double)(refsize * meret_merendo) / meret_merendo_refvanish < 2147483647 && (double)(refsize * meret_merendo) / meret_merendo_refvanish > -2147483647)) {
		/* ha a kapott ertek tul nagy, akkor hibaertekkel terunk vissza 
		   ekkor valsznleg nem voltak egy sikon a mrend objektumok, vagy
		   az egyik objektum tl kzel esett a vanishing line-hoz,
		   melynek kjvetkeztben tl nagy rtket kaptunk.*/
		return -1;
	}
	
	return (double)(refsize * meret_merendo) / meret_merendo_refvanish ;	//normlis esetben visszatrnk a kiszmolt rtkkel
}

/*
int main() {

	Measurement* proba = new Measurement(800,600,20,200,100,201);
	proba->setRefData(10,100,11,300,20);

	proba->setMeasureCoordinates(20,150,21,280);	//1. normlis eset	
	cout << proba->calculateSize() << endl;

	proba->setMeasureCoordinates(19,100,20,200);	//2. abnormlis eset
	cout << proba->calculateSize() << endl;

	proba->setMeasureCoordinates(19,100,20,300);	//3. prhuzamos eset
	cout << proba->calculateSize() << endl;

	delete proba;

	return 0;
}
*/
